function [mLifetime pLifetime] = flim_get_lifetimes_phasor(phasor, f)

%
% function [mLifetime pLifetime] = flim_get_lifetimes_phasor(phasor, f)
%
% This function computes the lifetimes based on the modulation index and
% phase at a given modulation frequency.
%
% Input parameters:
% phasor: 2D complex phasor image
% f:      Modulation frequency (in Hertz)
%
% Return values:
% mLifetime: 2D lifetime image based on the modulation index (in seconds)
% pLifetime: 2D lifetime image based on the phase (in seconds)
%

    mLifetime =  (1 / (2 * pi * f)) * sqrt(1 ./ abs(phasor) .^ 2 - 1);
    pLifetime = -(1 / (2 * pi * f)) * ...
                imag(phasor) ./ (real(phasor) + realmin);

end
